@@ -18,7 +18,14 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-#define LIBGNUTLS_VERSION "0.2.2"
+#ifndef GNUTLS_H
+# define GNUTLS_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+#define LIBGNUTLS_VERSION "0.2.3"
 
 #include <sys/types.h>
 
@@ -43,7 +50,7 @@ typedef enum AlertDescription { GNUTLS_CLOSE_NOTIFY, GNUTLS_UNEXPECTED_MESSAGE=1
 			} AlertDescription;
 
 
-typedef enum CertificateStatus { GNUTLS_CERT_TRUSTED=1, GNUTLS_CERT_NOT_TRUSTED, GNUTLS_CERT_EXPIRED, GNUTLS_CERT_INVALID } CertificateStatus;
+typedef enum CertificateStatus { GNUTLS_CERT_TRUSTED=1, GNUTLS_CERT_NOT_TRUSTED, GNUTLS_CERT_EXPIRED, GNUTLS_CERT_INVALID, GNUTLS_CERT_NONE } CertificateStatus;
 typedef enum CertificateRequest { GNUTLS_CERT_REQUEST=1, GNUTLS_CERT_REQUIRE } CertificateRequest;
 typedef enum CloseRequest { GNUTLS_SHUT_RDWR=0, GNUTLS_SHUT_WR=1 } CloseRequest;
 
@@ -112,7 +119,6 @@ int gnutls_set_mac_priority( GNUTLS_STATE state, LIST);
 int gnutls_set_compression_priority( GNUTLS_STATE state, LIST);
 int gnutls_set_kx_priority( GNUTLS_STATE state, LIST);
 int gnutls_set_protocol_priority( GNUTLS_STATE state, LIST);
-int gnutls_set_certificate_request( GNUTLS_STATE, CertificateRequest);
 
 /* set our version - 0 for TLS 1.0 and 1 for SSL3 */
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state);
@@ -220,10 +226,10 @@ void gnutls_global_set_recv_func( RECV_FUNC recv_func);
 typedef struct X509PKI_CLIENT_AUTH_INFO_INT *X509PKI_CLIENT_AUTH_INFO;
 typedef X509PKI_CLIENT_AUTH_INFO X509PKI_SERVER_AUTH_INFO;
 
-typedef struct SRP_CLIENT_AUTH_INFO_INT *SRP_CLIENT_AUTH_INFO;
-typedef struct SRP_SERVER_AUTH_INFO_INT *SRP_SERVER_AUTH_INFO;
-typedef struct ANON_CLIENT_AUTH_INFO_INT *ANON_CLIENT_AUTH_INFO;
-typedef struct ANON_SERVER_AUTH_INFO_INT *ANON_SERVER_AUTH_INFO;
+typedef struct SRP_CLIENT_AUTH_INFO_INT {} *SRP_CLIENT_AUTH_INFO;
+typedef struct SRP_SERVER_AUTH_INFO_INT {} *SRP_SERVER_AUTH_INFO;
+typedef struct ANON_CLIENT_AUTH_INFO_INT {} *ANON_CLIENT_AUTH_INFO;
+typedef struct ANON_SERVER_AUTH_INFO_INT {} *ANON_SERVER_AUTH_INFO;
 
 #ifndef GNUTLS_UI_H
 # define GNUTLS_UI_H
@@ -237,6 +243,7 @@ typedef struct ANON_SERVER_AUTH_INFO_INT *ANON_SERVER_AUTH_INFO;
 #define X509_OU_SIZE 256
 #define X509_L_SIZE 256
 #define X509_S_SIZE 256
+#define X509_EMAIL_SIZE 256
 
 typedef struct {
 	char common_name[X509_CN_SIZE];
@@ -245,6 +252,7 @@ typedef struct {
 	char organizational_unit_name[X509_OU_SIZE];
 	char locality_name[X509_L_SIZE];
 	char state_or_province_name[X509_S_SIZE];
+	char email[X509_EMAIL_SIZE];
 } gnutls_DN;
 
 /* For key Usage, test as:
@@ -277,24 +285,35 @@ int gnutls_anon_client_get_dh_bits(  ANON_CLIENT_AUTH_INFO info);
 
 /* X509PKI */
 
+int gnutls_set_x509_cert_callback( X509PKI_CREDENTIALS, int (*x509_cert_callback)(gnutls_DN*, gnutls_DN*, int, gnutls_DN*, int));
+int gnutls_x509pki_set_cert_request( GNUTLS_STATE, CertificateRequest);
 
-const gnutls_DN* gnutls_x509pki_client_get_peer_dn(  X509PKI_CLIENT_AUTH_INFO info);
-const gnutls_DN* gnutls_x509pki_client_get_issuer_dn(  X509PKI_CLIENT_AUTH_INFO info);
-CertificateStatus gnutls_x509pki_client_get_peer_certificate_status(  X509PKI_CLIENT_AUTH_INFO info);
-int gnutls_x509pki_client_get_peer_certificate_version(  X509PKI_CLIENT_AUTH_INFO info);
-time_t gnutls_x509pki_client_get_peer_certificate_activation_time(  X509PKI_CLIENT_AUTH_INFO info);
-time_t gnutls_x509pki_client_get_peer_certificate_expiration_time(  X509PKI_CLIENT_AUTH_INFO info);
-unsigned char gnutls_x509pki_client_get_key_usage(  X509PKI_CLIENT_AUTH_INFO info);
-const char* gnutls_x509pki_client_get_subject_alt_name(  X509PKI_CLIENT_AUTH_INFO info);
+const gnutls_DN* gnutls_x509pki_get_peer_dn(  X509PKI_CLIENT_AUTH_INFO info);
+const gnutls_DN* gnutls_x509pki_get_issuer_dn(  X509PKI_CLIENT_AUTH_INFO info);
+CertificateStatus gnutls_x509pki_get_peer_certificate_status(  X509PKI_CLIENT_AUTH_INFO info);
+int gnutls_x509pki_get_peer_certificate_version(  X509PKI_CLIENT_AUTH_INFO info);
+time_t gnutls_x509pki_get_peer_certificate_activation_time(  X509PKI_CLIENT_AUTH_INFO info);
+time_t gnutls_x509pki_get_peer_certificate_expiration_time(  X509PKI_CLIENT_AUTH_INFO info);
+unsigned char gnutls_x509pki_get_key_usage(  X509PKI_CLIENT_AUTH_INFO info);
+const char* gnutls_x509pki_get_subject_dns_name(  X509PKI_CLIENT_AUTH_INFO info);
 
-#define gnutls_x509pki_server_get_peer_dn gnutls_x509pki_client_get_peer_dn
-#define gnutls_x509pki_server_get_issuer_dn gnutls_x509pki_client_get_issuer_dn
-#define gnutls_x509pki_server_get_peer_certificate_status gnutls_x509pki_client_get_peer_certificate_status
-#define gnutls_x509pki_server_get_peer_certificate_version gnutls_x509pki_client_get_peer_certificate_version
-#define gnutls_x509pki_server_get_peer_certificate_activation_time gnutls_x509pki_client_get_peer_certificate_activation_time
-#define gnutls_x509pki_server_get_peer_certificate_expiration_time gnutls_x509pki_client_get_peer_certificate_expiration_time
-#define gnutls_x509pki_server_get_key_usage gnutls_x509pki_client_get_key_usage
-#define gnutls_x509pki_server_get_subject_alt_name gnutls_x509pki_client_get_subject_alt_name
+#define gnutls_x509pki_server_get_peer_dn gnutls_x509pki_get_peer_dn
+#define gnutls_x509pki_server_get_issuer_dn gnutls_x509pki_get_issuer_dn
+#define gnutls_x509pki_server_get_peer_certificate_status gnutls_x509pki_get_peer_certificate_status
+#define gnutls_x509pki_server_get_peer_certificate_version gnutls_x509pki_get_peer_certificate_version
+#define gnutls_x509pki_server_get_peer_certificate_activation_time gnutls_x509pki_get_peer_certificate_activation_time
+#define gnutls_x509pki_server_get_peer_certificate_expiration_time gnutls_x509pki_get_peer_certificate_expiration_time
+#define gnutls_x509pki_server_get_key_usage gnutls_x509pki_get_key_usage
+#define gnutls_x509pki_server_get_subject_dns_name gnutls_x509pki_get_subject_dns_name
+
+#define gnutls_x509pki_client_get_peer_dn gnutls_x509pki_get_peer_dn
+#define gnutls_x509pki_client_get_issuer_dn gnutls_x509pki_get_issuer_dn
+#define gnutls_x509pki_client_get_peer_certificate_status gnutls_x509pki_get_peer_certificate_status
+#define gnutls_x509pki_client_get_peer_certificate_version gnutls_x509pki_get_peer_certificate_version
+#define gnutls_x509pki_client_get_peer_certificate_activation_time gnutls_x509pki_get_peer_certificate_activation_time
+#define gnutls_x509pki_client_get_peer_certificate_expiration_time gnutls_x509pki_get_peer_certificate_expiration_time
+#define gnutls_x509pki_client_get_key_usage gnutls_x509pki_get_key_usage
+#define gnutls_x509pki_client_get_subject_dns_name gnutls_x509pki_get_subject_dns_name
 
 # endif /* LIBGNUTLS_VERSION */
 
@@ -534,8 +553,13 @@ int
 asn1_read_value(node_asn *root,char *name,unsigned char *value,int *len);
 
 int
-asn1_create_tree(static_asn *root,node_asn **pointer);
+asn1_create_tree(const static_asn *root,node_asn **pointer);
 
 #endif
 
 
+#ifdef __cplusplus
+}
+#endif
+#endif /* GNUTLS_H */
+
