@@ -172,6 +172,7 @@ int gnutls_deinit(GNUTLS_STATE state)
 
 	/* free priorities */
 	GNUTLS_FREE(state->gnutls_internals.MACAlgorithmPriority.algorithm_priority);
+	GNUTLS_FREE(state->gnutls_internals.ProtocolPriority.algorithm_priority);
 	GNUTLS_FREE(state->gnutls_internals.KXAlgorithmPriority.algorithm_priority);
 	GNUTLS_FREE(state->gnutls_internals.BulkCipherAlgorithmPriority.algorithm_priority);
 	GNUTLS_FREE(state->gnutls_internals.CompressionMethodPriority.algorithm_priority);
@@ -445,21 +446,23 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 #endif
 
 	for (i = 0; i < iterations; i++) {
-		cipher_size = _gnutls_encrypt( state, &data[i*Size], Size, &cipher, type);
+		cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, &data[i*Size], Size, &cipher, type);
 		if (cipher_size <= 0) {
 			gnutls_assert();
+			if (cipher_size==0) cipher_size = GNUTLS_E_ENCRYPTION_FAILED;
 			return cipher_size; /* error */
 		}
 		
-		WRITEuint16( cipher_size-RECORD_HEADER_SIZE, &headers[3]);
-		memcpy( cipher, headers, RECORD_HEADER_SIZE);
-
 		if (_gnutls_write(cd, cipher, cipher_size, flags) != cipher_size) {
+			gnutls_free( cipher);
 			state->gnutls_internals.valid_connection = VALID_FALSE;
 			state->gnutls_internals.resumable = RESUME_FALSE;
 			gnutls_assert();
 			return GNUTLS_E_UNABLE_SEND_DATA;
 		}
+
+		gnutls_free(cipher);
+
 #ifdef RECORD_DEBUG
 		_gnutls_log( "Record: Sended Packet[%d] %s(%d) with length: %d\n",
 		(int) uint64touint32(&state->connection_state.write_sequence_number), _gnutls_packet2str(type), type, cipher_size);
@@ -472,25 +475,31 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 			gnutls_assert();
 			return GNUTLS_E_RECORD_LIMIT_REACHED;
 		}
+
 	}
 
+
 	/* rest of data 
 	 */
 	if (iterations > 1) {
 		Size = sizeofdata % MAX_ENC_LEN;
-		cipher_size = _gnutls_encrypt( state, &data[i*Size], Size, &cipher, type);
-		if (cipher_size<=0) return cipher_size;
-
-		WRITEuint16( cipher_size-RECORD_HEADER_SIZE, &headers[3]);
-		memcpy( cipher, headers, RECORD_HEADER_SIZE);
-
+		cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, &data[i*Size], Size, &cipher, type);
+		if (cipher_size<=0) {
+			if (cipher_size == 0) cipher_size = GNUTLS_E_ENCRYPTION_FAILED;
+			gnutls_assert();
+			return cipher_size;
+		}
+		
 		if (_gnutls_write(cd, cipher, cipher_size, flags) != cipher_size) {
+			gnutls_free(cipher);
 			state->gnutls_internals.valid_connection = VALID_FALSE;
 			state->gnutls_internals.resumable = RESUME_FALSE;
 			gnutls_assert();
 			return GNUTLS_E_UNABLE_SEND_DATA;
 		}
 
+		gnutls_free(cipher);
+
 		/* increase sequence number
 		 */
 		if (uint64pp( &state->connection_state.write_sequence_number)!=0) {
@@ -502,8 +511,6 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 
 	ret += sizeofdata;
 
-	gnutls_free(cipher);
-
 	return ret;
 }
 
@@ -536,7 +543,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	uint8 *tmpdata;
 	int tmplen;
 	GNUTLS_Version version;
-	uint8 headers[RECORD_HEADER_SIZE];
+	uint8 *headers;
 	ContentType recv_type;
 	uint16 length;
 	uint8 *ciphertext;
@@ -574,7 +581,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	/* in order for GNUTLS_E_AGAIN to be returned the socket
 	 * must be set to non blocking mode
 	 */
-	if ( (ret = _gnutls_read_buffered(cd, state, headers, RECORD_HEADER_SIZE, flags, -1)) != RECORD_HEADER_SIZE) {
+	if ( (ret = _gnutls_read_buffered(cd, state, &headers, header_size, flags, -1)) != header_size) {
 		if (ret==GNUTLS_E_AGAIN) return ret;
 
 		state->gnutls_internals.valid_connection = VALID_FALSE;
@@ -653,12 +660,9 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
-	recv_data = gnutls_malloc(length+header_size);
-
 	/* check if we have that data into buffer. 
  	 */
-	if ( (ret = _gnutls_read_buffered(cd, state, recv_data, header_size+length, flags, recv_type)) != length+header_size) {
-		gnutls_free(recv_data);
+	if ( (ret = _gnutls_read_buffered(cd, state, &recv_data, header_size+length, flags, recv_type)) != length+header_size) {
 		if (ret==GNUTLS_E_AGAIN) return ret;
 
 		state->gnutls_internals.valid_connection = VALID_FALSE;
@@ -677,7 +681,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	 */
 	tmplen = _gnutls_decrypt( state, ciphertext, length, &tmpdata, recv_type);
 	if (tmplen < 0) {
-		switch (tmplen) {
+		switch (tmplen) { /* send appropriate alert */
 			case GNUTLS_E_MAC_FAILED:
 				gnutls_send_alert(cd, state, GNUTLS_FATAL, GNUTLS_BAD_RECORD_MAC);
 				break;
@@ -691,7 +695,6 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		state->gnutls_internals.valid_connection = VALID_FALSE;
 		state->gnutls_internals.resumable = RESUME_FALSE;
 		gnutls_assert();
-		gnutls_free(recv_data);
 		return tmplen;
 	}
 
@@ -702,8 +705,6 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		_gnutls_log( "Record: ChangeCipherSpec Packet was received\n");
 #endif
 
-		gnutls_free(recv_data);
-
 		if (tmplen!=sizeofdata) { /* sizeofdata should be 1 */
 			gnutls_assert();
 			gnutls_free(tmpdata);
@@ -728,8 +729,6 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		return GNUTLS_E_RECORD_LIMIT_REACHED;
 	}
 
-	gnutls_free(recv_data);
-
 	if ( (recv_type == type) && (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE)) {
 		gnutls_insertDataBuffer(type, state, (void *) tmpdata, tmplen);
 	} else {
