@@ -30,6 +30,14 @@
 #include "../lib/gnutls.h"
 #include <sys/time.h>
 
+#ifndef SHUT_WR
+# define SHUT_WR 1
+#endif
+
+#ifndef SHUT_RDWR
+# define SHUT_RDWR 2
+#endif
+
 #define SA struct sockaddr
 #define ERR(err,s) if (err==-1) {perror(s);return(1);}
 #define MAX_BUF 4096
@@ -49,7 +57,8 @@
 	PRINTX( "L:", X->locality_name); \
 	PRINTX( "S:", X->state_or_province_name); \
 	PRINTX( "C:", X->country); \
-	PRINTX( "SAN:", gnutls_x509pki_client_get_subject_alt_name(x509_info))
+	PRINTX( "E:", X->email); \
+	PRINTX( "SAN:", gnutls_x509pki_client_get_subject_dns_name(x509_info))
 
 static int print_info( GNUTLS_STATE state) {
 const char *tmp;
@@ -79,6 +88,9 @@ const gnutls_DN* dn;
 			case GNUTLS_CERT_TRUSTED:
 				printf("- Peer's X509 Certificate was verified\n");
 				break;
+			case GNUTLS_CERT_NONE:
+				printf("- Peer did not send any X509 Certificate.\n");
+				break;
 			case GNUTLS_CERT_INVALID:
 			default:
 				printf("- Peer's X509 Certificate was invalid\n");
@@ -113,6 +125,34 @@ const gnutls_DN* dn;
 	return 0;
 }
 
+int cert_callback( gnutls_DN *client_cert, gnutls_DN *issuer_cert, int ncerts, gnutls_DN* req_ca_cert, int nreqs) {
+
+	if (client_cert==NULL) {
+		return 0; /* means the we will only be called again
+		           * if the library cannot determine which
+		           * certificate to send
+		           */
+	}
+
+#if 0
+	/* here we should prompt the user and ask him
+	 * which certificate to choose. Too bored to 
+	 * implement that. --nmav
+	 */
+	for (i=0;i<ncerts;i++){
+		fprintf(stderr, "%s.", client_cert->common_name);
+		fprintf(stderr, "%s\n", issuer_cert->common_name);
+	}
+	for (i=0;i<nreqs;i++){
+		fprintf(stderr, "%s.", req_ca_cert->common_name);
+	}
+	fprintf(stderr, "\n");
+	return 0;
+#endif
+
+	return -1; /* send no certificate to the peer */
+}
+
 int main(int argc, char** argv)
 {
 	int err, ret;
@@ -152,6 +192,7 @@ int main(int argc, char** argv)
 	}
 	gnutls_set_x509_client_trust( xcred, CAFILE, CRLFILE);
 	gnutls_set_x509_client_key( xcred, CLICERTFILE, CLIKEYFILE);
+	gnutls_set_x509_cert_callback( xcred, cert_callback);
 
 	/* SRP stuff */
 	if (gnutls_allocate_srp_client_sc( &cred)<0) {
@@ -175,7 +216,7 @@ int main(int argc, char** argv)
 	gnutls_init(&state, GNUTLS_CLIENT);
 	
 	gnutls_set_protocol_priority( state, GNUTLS_TLS1, GNUTLS_SSL3, 0);
-	gnutls_set_cipher_priority( state, GNUTLS_ARCFOUR, GNUTLS_3DES_CBC, GNUTLS_RIJNDAEL_CBC, 0);
+	gnutls_set_cipher_priority( state, GNUTLS_3DES_CBC, GNUTLS_RIJNDAEL_CBC, 0);
 	gnutls_set_compression_priority( state, GNUTLS_ZLIB, GNUTLS_NULL_COMPRESSION, 0);
 	gnutls_set_kx_priority( state, GNUTLS_KX_RSA, GNUTLS_KX_SRP, GNUTLS_KX_DH_ANON, 0);
 	gnutls_set_mac_priority( state, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5, 0);
@@ -230,7 +271,7 @@ int main(int argc, char** argv)
 	gnutls_init(&state, GNUTLS_CLIENT);
 	
 	gnutls_set_protocol_priority( state, GNUTLS_TLS1, GNUTLS_SSL3, 0);
-	gnutls_set_cipher_priority( state, GNUTLS_ARCFOUR, GNUTLS_3DES_CBC, GNUTLS_TWOFISH_CBC, GNUTLS_RIJNDAEL_CBC, 0);
+	gnutls_set_cipher_priority( state, GNUTLS_3DES_CBC, GNUTLS_TWOFISH_CBC, GNUTLS_RIJNDAEL_CBC, 0);
 	gnutls_set_compression_priority( state, GNUTLS_NULL_COMPRESSION, 0);
 	gnutls_set_kx_priority( state, GNUTLS_KX_RSA, GNUTLS_KX_SRP, GNUTLS_KX_DH_ANON, 0);
 
