@@ -76,14 +76,6 @@ GNUTLS_STATE initialize_state()
 	int ret;
 
 
-	/* this is a password file (created with the included crypt utility) 
-	 * Read README.crypt prior to using SRP.
-	 */
-	gnutls_allocate_srp_server_sc( &srp_cred);
-	gnutls_set_srp_server_cred( srp_cred, SRP_PASSWD, SRP_PASSWD_CONF);
-
-	gnutls_allocate_anon_server_sc( &dh_cred);
-	gnutls_set_anon_server_cred( dh_cred, 1024);
 
 	gnutls_init(&state, GNUTLS_SERVER);
 	if ((ret = gnutls_set_db_name(state, "gnutls-rsm.db")) < 0)
@@ -105,7 +97,7 @@ GNUTLS_STATE initialize_state()
 
 	gnutls_set_mac_priority(state, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5, 0);
 
-	gnutls_set_certificate_request( state, GNUTLS_CERT_REQUEST);
+	gnutls_x509pki_set_cert_request( state, GNUTLS_CERT_REQUEST);
 
 	return state;
 }
@@ -117,7 +109,8 @@ GNUTLS_STATE initialize_state()
 	PRINTX( "L:", X->locality_name); \
 	PRINTX( "S:", X->state_or_province_name); \
 	PRINTX( "C:", X->country); \
-	PRINTX( "SAN:", gnutls_x509pki_client_get_subject_alt_name(x509_info))
+	PRINTX( "E:", X->email); \
+	PRINTX( "SAN:", gnutls_x509pki_client_get_subject_dns_name(x509_info))
 
 void print_info(GNUTLS_STATE state)
 {
@@ -168,6 +161,9 @@ void print_info(GNUTLS_STATE state)
 			case GNUTLS_CERT_TRUSTED:
 				printf("- Peer's X509 Certificate was verified\n");
 				break;
+			case GNUTLS_CERT_NONE:
+				printf("- Peer did not send any certificate.\n");
+				break;
 			case GNUTLS_CERT_INVALID:
 			default:
 				printf("- Peer's X509 Certificate was invalid\n");
@@ -179,11 +175,14 @@ void print_info(GNUTLS_STATE state)
 			printf(" - Certificate version: #%d\n", gnutls_x509pki_client_get_peer_certificate_version(x509_info));
 
 			dn = gnutls_x509pki_client_get_peer_dn( x509_info);
-			PRINT_DN( dn);
+			if (dn!=NULL)
+				PRINT_DN( dn);
 
 			dn = gnutls_x509pki_client_get_issuer_dn( x509_info);
-			printf(" - Certificate Issuer's info:\n");
-			PRINT_DN( dn);
+			if (dn!=NULL) {
+				printf(" - Certificate Issuer's info:\n");
+				PRINT_DN( dn);
+			}
 		}
 	}
 
@@ -371,7 +370,14 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	/* this is a password file (created with the included crypt utility) 
+	 * Read README.crypt prior to using SRP.
+	 */
+	gnutls_allocate_srp_server_sc( &srp_cred);
+	gnutls_set_srp_server_cred( srp_cred, SRP_PASSWD, SRP_PASSWD_CONF);
 
+	gnutls_allocate_anon_server_sc( &dh_cred);
+	gnutls_set_anon_server_cred( dh_cred, 1024);
 
 	listen_sd = socket(AF_INET, SOCK_STREAM, 0);
 	ERR(listen_sd, "socket");
@@ -391,6 +397,7 @@ int main(int argc, char **argv)
 	printf("%s ready. Listening to port '%d'.\n\n", name, PORT);
 
 	client_len = sizeof(sa_cli);
+	
 	for (;;) {
 		state = initialize_state();
 
@@ -462,7 +469,7 @@ int main(int argc, char **argv)
 		 */
 		close(sd);
 		gnutls_deinit(state);
-		
+
 	}
 	close(listen_sd);
 
